@@ -1,8 +1,8 @@
 package HTML::Template::Compiled;
-# $Id: Compiled.pm 1137 2011-11-21 19:36:03Z tinita $
+# $Id: Compiled.pm 1161 2012-05-05 14:00:22Z tinita $
 # doesn't work with make tardist
 #our $VERSION = ($version_pod =~ m/^\$VERSION = "(\d+(?:\.\d+)+)"/m) ? $1 : "0.01";
-our $VERSION = "0.96";
+our $VERSION = "0.97";
 use Data::Dumper;
 BEGIN {
 use constant D => $ENV{HTC_DEBUG} || 0;
@@ -59,7 +59,7 @@ BEGIN {
           debug debug_file objects perl out_fh default_escape
           filter formatter
           globalstack use_query parse_tree parser compiler includes
-          plugins open_mode
+          plugins open_mode chomp
         )
           #use_expressions
     );
@@ -354,6 +354,7 @@ sub from_cache {
             $t->set_plugins($plug) if @$plug;
             return $t;
         }
+#        warn __PACKAGE__.':'.__LINE__.": not in mem cache: $fname\n";
     }
     D && $self->log( "from_cache() 2 filename=" . $self->get_filename );
 
@@ -766,7 +767,7 @@ sub include_file {
 }
 
 sub createFilename {
-    my ( $self, $path, $filename ) = @_;
+    my ( $self, $path, $filename, $cwd ) = @_;
     D && $self->log("createFilename($path,$filename)");
     D && $self->stack(1);
     if ($path) {
@@ -782,7 +783,9 @@ sub createFilename {
     else {
         D && $self->log( "file: " . File::Spec->catfile( $path, $filename ) );
         if ($path && @$path) {
-            for ( @$path ) {
+            my @search = @$path;
+            push @search, $cwd if defined $cwd;
+            for ( @search ) {
                 #if (ref $_) {
                 #    my $foo = basename $$_;
                 #    my ($dir, $f) = (dirname($$_), basename($$_));
@@ -878,6 +881,8 @@ sub init {
         use_perl               => 0,
         open_mode              => '',
         no_includes            => 0,
+        pre_chomp              => 0,
+        post_chomp             => 0,
         %args,
     );
     $self->set_loop_context(1) if $args{loop_context_vars};
@@ -885,6 +890,7 @@ sub init {
     $self->set_default_escape( $defaults{default_escape} );
     $self->set_default_path( $defaults{default_path} );
     $self->set_use_query( $defaults{use_query} );
+    $self->set_chomp([$args{pre_chomp}, $args{post_chomp}]);
     #$self->set_use_expressions( $defaults{use_expressions} );
     if ($defaults{use_expressions}) {
         require HTML::Template::Compiled::Expr;
@@ -927,6 +933,7 @@ sub init {
         $parser->set_perl($defaults{use_perl});
         $parser->set_expressions($defaults{use_expressions});
     }
+    $parser->set_chomp([$args{pre_chomp}, $args{post_chomp}]);
     if ($defaults{use_perl}) {
         $parser->add_tagnames({
             HTML::Template::Compiled::Token::OPENING_TAG() => {
@@ -1040,7 +1047,7 @@ sub quote_file {
 # hash.key (found inside <tmpl_var name="hash.key">) will be converted to
 # '$t->get_var($P, $$C, 1, [PATH_DEREF, 'key'])'
 # the get_var method walks the paths given through the data structure.
-# $P is the paramater hash of the template, $C is a reference to the current
+# $P is the parameter hash of the template, $C is a reference to the current
 # parameter hash. the third argument to get_var is 'final'.
 # <tmpl_var foo> is a 'final' path, and <tmpl_with foo> is not.
 # so final means it's in 'print-context'.
@@ -1573,7 +1580,7 @@ HTML::Template::Compiled - Template System Compiles HTML::Template files to Perl
 
 =head1 VERSION
 
-$VERSION = "0.96"
+$VERSION = "0.97"
 
 =cut
 
@@ -1581,7 +1588,7 @@ sub __test_version {
     my $v = __PACKAGE__->VERSION;
     my ($v_test) = $version_pod =~ m/VERSION\s*=\s*"(.+)"/m;
     no warnings;
-    return $v == $v_test ? 1 : 0;
+    return $v eq $v_test ? 1 : 0;
 }
 
 1;
@@ -1601,7 +1608,7 @@ __END__
 
   # note that the following
   # use HTML::Template::Compiled speed => 1
-  # is deprecated (can be problematic under persistant environments)
+  # is deprecated (can be problematic under persistent environments)
 
   # or for the biggest compatibility with HTML::Template
   # case_sensitive => 0
@@ -1609,7 +1616,7 @@ __END__
   # use_query => 1
   # note that the following
   # use HTML::Template::Compiled compatible => 1;
-  # is deprecated (can be problematic under persistant environments)
+  # is deprecated (can be problematic under persistent environments)
 
   # or use HTML::Template::Compiled::Classic
 
@@ -1633,29 +1640,37 @@ __END__
   Title: <TMPL_VAR TITLE> (<TMPL_VAR YEAR>)
   </TMPL_LOOP>
 
+  Or use different tag styles:
+  Band: <%= BAND %>
+  <%loop ALBUMS %>
+  Title: <%= TITLE %> (<%= YEAR %>)
+  <%/loop %>
+  Band: [%= BAND %]
+  [%loop ALBUMS %]
+  Title: [%= TITLE %] ([%= YEAR %])
+  [%/loop %]
+
 =head1 DESCRIPTION
 
+HTML::Template::Compiled is a template system which can be used for
+L<HTML::Template> templates with the same API. It
+offers more flexible template delimiters, additional tags and features,
+and by compiling the template into perl code it can run significantly faster
+in persistent environments such as FastCGI or mod_perl.
+
 For a quick reference, see L<HTML::Template::Compiled::Reference>.
-HTML::Template::Compiled is a templating module which can be used
-for creating HTML but also plaintext or other output.
 
 As the basic features work like in L<HTML::Template>, please get familiar
 with its documentation before.
 
 HTML::Template::Compiled (HTC) does not implement all features of
-L<HTML::Template>, and
+L<HTML::Template> (see L<"COMPATIBILITY">), and
 it has got some additional features which are explained below:
 L<"ADDITIONAL FEATURES">
 
-HTML::Template::Compiled (HTC) is a template system which uses the same
-template syntax as HTML::Template and the same perl API (see L<"COMPATIBILITY">
-for what you need to know if you want (almost) the same behaviour). Internally
-it works different, because it turns the template into perl code,
-and once that is done, generating the output is much faster than with
-HTML::Template (please see L<"BENCHMARKS"> for some examples, because a
-comparison depends on so much parameters).
-But you should run in a persistent environment like mod_perl oder FastCGI,
-otherwise it might be even slower.
+See L<"BENCHMARKS"> for some examples on the performance. Since it depends
+highly on the options used and on the template size there can be no general
+statement on its performance.
 
 You might want to use L<HTML::Template::Compiled::Lazy> for CGI environments
 as it doesn't parse the template before calling output. But note that HTC::Lazy
@@ -1731,23 +1746,31 @@ What can HTC do for you additionally to HTML::Template?
 
 =over 4
 
-=item TMPL_ELSIF
+=item tag TMPL_ELSIF
 
 No need to have cascading "if-else-if-else"s
 
-=item TMPL_EACH
+=item tag TMPL_EACH
 
 Iterate over a hash. See L<"TMPL_EACH">
 
-=item TMPL_WITH
+=item tag TMPL_WITH
 
 see L<"TMPL_WITH">
 
-=item TMPL_WHILE
+=item tag TMPL_WHILE
 
 see L<"TMPL_WHILE">
 
-=item TMPL_COMMENT, TMPL_NOPARSE, TMPL_VERBATIM
+=item tag TMPL_SET_VAR
+
+see L<"SET_VAR">
+
+=item tag TMPL_USE_VARS
+
+see L<"USE_VARS">
+
+=item tags TMPL_COMMENT, TMPL_NOPARSE, TMPL_VERBATIM
 
 see L<"TMPL_COMMENT">, L<"TMPL_NOPARSE">, L<"TMPL_VERBATIM">
 
@@ -1772,7 +1795,7 @@ will turn out as:
 
 See also option debug_file in L<"OPTIONS"> for adding the filename globally.
 
-=item TMPL_SWITCH, TMPL_CASE
+=item tags TMPL_SWITCH, TMPL_CASE
 
 see L<"TMPL_SWITCH">
 
@@ -1780,27 +1803,30 @@ see L<"TMPL_SWITCH">
 
 Include perl code in your template. See L<"RUNNING PERL WITH TMPL_PERL">
 
-=item Chomp
+=item CHOMP
 
-Experimental feature, added in version 0.89. By using special tags the
-newlines before and/or after the tags will be deleted.
-I'm not sure about the syntax, so this might change. I'd be very glad about
-comments. Example:
+New in version 0.96_001, please report any bugs and send me suggestions.
+
+You can set global chomp options in the constructor. These work like in
+Template-Toolkit:
 
     my $htc = HTML::Template::Compiled->new(
-        ...
-        tagstyle => [qw/+asp_chomp/], # classic_chomp tt_chomp comment_chomp
+        pre_chomp  => 0, # 0, 1, 2, 3, default 0
+        post_chomp => 1, # 0, 1, 2, 3, default 0
     );
-    template:
-    <+-%= foo %>
-    bar
-    <-+%= baz %>
-    is the same as
-    <%= foo %>bar<%= baz %>
 
-So C<+-> means, leave the newline in front alone, but chomp after it,
-C<-+> is the opposite, C<--> chomps both and C<++> is just a no-op and
-behaves like a normal tag.
+Meaning of the values:
+0: Don't chomp
+1: remove only spaces in the line before or after the tag
+2: remove all whitespaces before or after the tag, and replace with one space
+3: remove all whitespaces before or after the tag
+
+In the template you can change that feature by using PRE_CHOMP and POST_CHOMP
+attributes:
+
+    <%= foo PRE_CHOMP=3 POST_CHOMP=1 %>
+
+The experimental tags +..._chomp have been removed.
 
 =item Generating perl code
 
@@ -1822,7 +1848,7 @@ See L<"OPTIONS">
 
 C<INCLUDE_VAR>, C<INCLUDE_STRING>. See L<"INCLUDE">
 
-=item TMPL_IF_DEFINED
+=item tag TMPL_IF_DEFINED
 
 Check for definedness instead of truth:
   <TMPL_IF_DEFINED NAME="var">
@@ -1856,6 +1882,10 @@ can use E<lt>% %E<gt> tags and the E<lt>%= tag instead of E<lt>%VAR (which will
 Define your own tagstyles and/or deactivate predefined ones.
 See L<"OPTIONS"> tagstyle.
 
+=item pre_chomp, post_chomp
+
+See L<"CHOMP">
+
 =back
 
 =head2 MISSING FEATURES
@@ -1894,10 +1924,10 @@ tags are different:
     <tmpl_var fOO> prints the value of hash key 'fOO'
 
 With case_sensitive set to 0, all your parameters passed to C<param()>
-are converted to uppercase, and the following tags are the same:
+are converted to lowercase, and the following tags are the same:
 
-    <tmpl_var Foo> prints the value of hash key 'FOO'
-    <tmpl_var fOO> prints the value of hash key 'FOO'
+    <tmpl_var Foo> prints the value of hash key 'foo'
+    <tmpl_var fOO> prints the value of hash key 'foo'
 
 
 =item subref variables
@@ -1921,12 +1951,10 @@ default is 0 (off). Set it via
 
 If you want to have your templates read in utf-8, use
 
-    open_mode => ':utf8',
+    open_mode => ':encoding(utf-8)',
 
 as an option.
 
-In the previous version, it was '<:utf8'. This is deprecated.
-
 =back
 
 Note: the following is deprecated:
@@ -2064,7 +2092,7 @@ or C<ALBUMS[0].SONGS[0].NAME> (the latter has changed since version 0.79)
 
 This is still in development, so I might change the API here.
 
-Additionally to feeding a simple hash do HTC, you can feed it objects.
+Additionally to feeding a simple hash to HTC, you can feed it objects.
 To do method calls you can also use '.' in the template.
 
   my $htc = HTML::Template::Compiled->new(
@@ -2233,7 +2261,7 @@ Inside TMPL_WITH you can't reference parent nodes unless you're using global_var
 
 =head2 TMPL_LOOP
 
-The special name C<_> gives you the current paramater. In loops you can use it like this:
+The special name C<_> gives you the current parameter. In loops you can use it like this:
 
  <tmpl_loop foo>
   Current item: <tmpl_var _ >
@@ -2331,6 +2359,40 @@ TMPL_EACH expects a ref), then you can set C<context=list>:
     <tmpl_var __key__ >
     </tmpl_each>
 
+=head2 SET_VAR
+
+Since 0.96_002
+
+Sets a local variable to the value given in C<value> or C<expr>
+
+    <tmpl_set foo expr=23>
+    <tmpl_set name=bar expr=23>
+    <tmpl_set boo value=var.boo>
+    <tmpl_set oof expr="21*2">
+    <tmpl_var foo>
+    <tmpl_var bar>
+    ...
+
+C<value=..> behaves like a variable name from the parameter stash.
+The variable name to set must match /[0-9a-z_]+/i
+
+Note that you cannot use those vars by default in includes.
+If you set a var with SET_VAR and want to use it in an include, use must
+use the L<"USE_VARS"> tag.
+
+=head2 USE_VARS
+
+Necessary if you want vars like SET_VAR and loop aliases from outside
+in includes.
+Before the first use in the include, add:
+
+    <tmpl_use_vars foo,bar,boo >
+
+so that the compiler recognizes them as user defined vars and not parameters
+from the stash.
+This statement is valid until the end of the template so you cannot
+"overwrite" parameters of the stash locally.
+
 =head2 TMPL_COMMENT
 
 For debugging purposes you can temporarily comment out regions:
@@ -2694,13 +2756,16 @@ tt-style and your own C<{{}} > style, then say:
 
 =item use_expressions (since 0.91_003)
 
-Set to 1 if you want to use expressions. They work more or less like
-in L<HTML::Template::Expr> - I took the parsing code from it and
+Set to 1 if you want to use expressions. The basic expressions work more or
+less like in L<HTML::Template::Expr> - I took the parsing code from it and
 used it with some minor changes - thanks to Sam Tregar.
 
     <%if expr="some.var > 3" %>It's grater than 3<%/if %>
 
-Additionally you can use object methods with parameters. While a
+But with expressions you can also use more complex navigation through the
+template stash:
+
+You can use object methods with parameters. While a
 normal method call can only be called without parameters, like
 
     <%= object.name %>
@@ -2709,12 +2774,45 @@ with expressions you can give it parameters:
 
     <%= expr="object.create_link('navi')" %>
 
-Inside function and method calls you also can use template
-vars.
+Inside function and method calls, hash keys you also can use template
+vars (array indices and hash keys since 0.96_003).
+
+    <%= expr=".path.to.hash{var}" %>
+    <%= expr=".path.to.hash{.another.var[123]}{'literal key'}" %>
 
 It is only minimally tested yet, so use with care and please report any
 bugs you find.
 
+A useful example: Output a number of items with their prices formatted.
+
+    my $nf = Number::Format->new(...);
+    my $htc = HTML::Template::Compiled->new(
+        filename => 'items.html',
+        use_expressions => 1,
+    );
+    $htc->param(
+        items => [
+            { size =>  50 * 1024 * 1024 * 1024, price => 49.95 },
+            { size => 250 * 1024 * 1024 * 1024, price => 110.99 },
+        ],
+        nf => $nf,
+    );
+
+items.html:
+
+    <%loop .items %>
+    Size: <%= expr=".nf.format_bytes(size)" %>
+    Price: <%= expr=".nf.format_price(price)" %>
+    <%/loop %>
+
+Output:
+    Size: 50G
+    Price: 49,95 EUR 
+    
+    Size: 250G
+    Price: 110,99 EUR 
+
+
 =item formatter
 
 Deprecated, see L<HTML::Template::Compiled::Formatter> please.
@@ -3110,7 +3208,7 @@ Corion, Limbic~Region, tye, runrig and others from perlmonks.org
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005 by Tina Mueller
+Copyright (C) 2005-2012 by Tina Mueller
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.3 or,
