@@ -1,5 +1,5 @@
 package HTML::Template::Compiled::Compiler;
-# $Id: Compiler.pm 1132 2011-11-12 14:26:03Z tinita $
+# $Id: Compiler.pm 1161 2012-05-05 14:00:22Z tinita $
 use strict;
 use warnings;
 use Data::Dumper;
@@ -8,7 +8,7 @@ use HTML::Template::Compiled::Expression qw(:expressions);
 use HTML::Template::Compiled::Utils qw(:walkpath);
 use File::Basename qw(dirname);
 
-our $VERSION = '0.17';
+our $VERSION = '0.18';
 
 use Carp qw(croak carp);
 use constant D             => 0;
@@ -29,6 +29,8 @@ use constant T_WHILE       => 'WHILE';
 use constant T_EACH        => 'EACH';
 use constant T_INCLUDE_VAR => 'INCLUDE_VAR';
 use constant T_INCLUDE_STRING => 'INCLUDE_STRING';
+use constant T_USE_VARS    => 'USE_VARS';
+use constant T_SET_VAR     => 'SET_VAR';
 
 use constant INDENT        => '    ';
 
@@ -189,7 +191,8 @@ sub parse_var {
         );
     }
     if ( grep { defined $_ && $args{var} eq $_ } @$lexicals ) {
-        return "\$$args{var}";
+        my $varstr = "\$lexi_$args{var}";
+        return $varstr;
     }
     my $lexi = join '|', grep defined, @$lexicals;
     my $varname = '$var';
@@ -205,7 +208,7 @@ sub parse_var {
         }
     }
     if ($lexi and $args{var} =~ s/^($lexi)($re)/$2/) {
-        $initial_var = "\$$1";
+        $initial_var = "\$lexi_$1";
     }
     elsif ( $args{var} =~ m/^_/ && $args{var} !~ m/^__(\w+)__$/ ) {
         $args{var} =~ s/^_//;
@@ -449,7 +452,9 @@ EOM
         formatter_path => $format,
         lexicals       => \@lexicals,
     );
+    my %use_vars;
     for my $token (@p) {
+        @use_vars{ @lexicals } = () if @lexicals;
         my ($text, $line, $open_close, $tname, $attr, $f, $nlevel) = @$token;
         #print STDERR "tags: ($text, $line, $open_close, $tname, $attr)\n";
         #print STDERR "p: '$text'\n";
@@ -535,6 +540,69 @@ ${indent}    if (defined \$\$C) {
 EOM
         }
 
+        if ( $tname eq T_USE_VARS ) {
+            my $vars     = $attr->{NAME};
+            my @l = grep length, split /\s*,\s*/, $vars;
+            for my $var (@l) {
+                if ($var =~ tr/a-zA-Z0-9_//c) {
+                    $self->get_parser->_error_wrong_tag_syntax(
+                        {
+                            fname => $token->get_file,
+                            line  => $token->get_line,
+                            token => "",
+                        },
+                        $var,
+                        'invalid SET_VAR/USE_VARS var name',
+                    );
+                }
+            }
+            push @lexicals, @l;
+        }
+        elsif ( $tname eq T_SET_VAR ) {
+            my $var     = $attr->{NAME};
+            if ($var =~ tr/a-zA-Z0-9_//c) {
+                $self->get_parser->_error_wrong_tag_syntax(
+                    {
+                        fname => $token->get_file,
+                        line  => $token->get_line,
+                        token => "",
+                    },
+                    $var,
+                    'invalid SET_VAR/USE_VARS var name',
+                );
+            }
+            my $value;
+            my $expr;
+            if (exists $attr->{VALUE}) {
+                $value = $attr->{VALUE};
+            }
+            elsif (exists $attr->{EXPR}) {
+                $expr = $attr->{EXPR};
+            }
+            else {
+                $self->get_parser->_error_wrong_tag_syntax(
+                    {
+                        fname => $token->get_file,
+                        line  => $token->get_line,
+                        token => "",
+                    },
+                    $var,
+                    'missing VALUE or EXPR',
+                );
+            }
+
+            unshift @lexicals, $var;
+            my $varstr = $class->parse_var($self,
+                %var_args,
+                var         => $value,
+                context     => $token,
+                compiler    => $class,
+                expr        => $expr,
+            );
+            $code .= <<"EOM";
+${indent}local \$lexi_$var = $varstr;
+EOM
+        }
         # --------- TMPL_LOOP TMPL_WHILE TMPL_EACH
         elsif ( ($tname eq T_LOOP || $tname eq T_WHILE || $tname eq T_EACH) ) {
             my $var     = $attr->{NAME};
@@ -584,7 +652,7 @@ EOM
             }
             my $global = '';
             my $lexi =
-              defined $lexical ? "${indent}my \$$lexical = \$\$C;\n" : "";
+              defined $lexical ? "${indent}local \$lexi_$lexical = \$\$C;\n" : "";
             if ($self->get_global_vars) {
                 my $pop_global = _expr_method(
                     'pushGlobalstack',
@@ -793,13 +861,15 @@ EOM
                     $info_stack->[-1]->{lc $filename}->{type} = $tname;
                 }
                 $varstr   = $self->quote_file($filename);
+                my $cwd;
 				unless ($self->get_scalar) {
 					$dir      = dirname($fname);
 					if ($self->get_search_path) {
 						if ( defined $dir and !grep { $dir eq $_ } @$path ) {
+                            $cwd = $dir;
 							# add the current directory to top of paths
 							# create new $path, don't alter original ref
-							$path = [ $dir, @$path ] ;
+#							$path = [ $dir, @$path ] ;
 						}
 					}
 					else {
@@ -812,7 +882,7 @@ EOM
                     #warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$self->get_file], ['file']);
                     #warn __PACKAGE__.':'.__LINE__.$".Data::Dumper->Dump([\$path], ['path']);
                     #$fullpath = $self->createFilename( [@$path, \$self->get_file], $filename );
-                    $fullpath = $self->createFilename( [@$path], $filename );
+                    $fullpath = $self->createFilename( [@$path], $filename, $cwd );
                     my $recursed = ++$HTML::Template::Compiled::COMPILE_STACK{$fullpath};
                     #warn __PACKAGE__." fullpath $fullpath ($recursed)\n";
                     if ($recursed <= 1) {
@@ -946,6 +1016,10 @@ EOM
     if ($self->get_use_query) {
         $self->set_parse_tree($info);
     }
+    my @use_vars = grep length, keys %use_vars;
+    if (@use_vars) {
+        $header .= qq#use vars qw/ @{[ map { '$lexi_'.$_ } @use_vars ]} /;\n#;
+    }
     #warn Data::Dumper->Dump([\$info], ['info']);
     $code .= qq#return \$OUT;\n#;
     $code = $header . $code . "\n} # end of sub\n";
